@@ -40,6 +40,9 @@ void pygdk_add_constants(PyObject *module, const gchar *strip_prefix);
 extern PyMethodDef pygtk_functions[];
 extern PyMethodDef pygdk_functions[];
 
+PyObject *PyGtkDeprecationWarning;
+PyObject *PyGtkWarning;
+
 static struct _PyGtk_FunctionStruct functions = {
     VERSION,
 
@@ -47,6 +50,24 @@ static struct _PyGtk_FunctionStruct functions = {
     pygdk_rectangle_from_pyobject,
 };
 
+static void
+sink_gtkwindow(GObject *object)
+{
+    if (object->ref_count == 1
+	&& GTK_WINDOW(object)->has_user_ref_count) {
+	g_object_ref(object);
+    }
+}
+
+static void
+sink_gtkinvisible(GObject *object)
+{
+    if (object->ref_count == 1
+	&& GTK_INVISIBLE(object)->has_user_ref_count) {
+	g_object_ref(object);
+    }
+}
+
 static void
 sink_gtkobject(GObject *object)
 {
@@ -56,6 +77,15 @@ sink_gtkobject(GObject *object)
     }
 }
 
+static void
+_pygtk_log_func(const gchar *log_domain,
+                GLogLevelFlags log_level,
+                const gchar *message,
+                gpointer user_data)
+{
+    PyErr_Warn(PyGtkWarning, (char *) message);
+}
+
 static gboolean
 python_do_pending_calls(gpointer data)
 {
@@ -88,6 +118,8 @@ init_gtk(void)
     init_pygobject();
     g_assert(pygobject_register_class != NULL);
 
+    pygobject_register_sinkfunc(GTK_TYPE_WINDOW, sink_gtkwindow);
+    pygobject_register_sinkfunc(GTK_TYPE_INVISIBLE, sink_gtkinvisible);
     pygobject_register_sinkfunc(GTK_TYPE_OBJECT, sink_gtkobject);
 
     /* set the default python encoding to utf-8 */
@@ -152,6 +184,10 @@ init_gtk(void)
 			 o=PyCObject_FromVoidPtr(&functions, NULL));
     Py_DECREF(o);
 	
+    PyGtkDeprecationWarning = PyErr_NewException("gtk.GtkDeprecationWarning",
+						 PyExc_DeprecationWarning, NULL);
+    PyDict_SetItemString(d, "DeprecationWarning", PyGtkDeprecationWarning);
+
     stock_ids = gtk_stock_list_ids();
     strcpy(buf, "STOCK_");
     for (cur = stock_ids; cur; cur = stock_ids) {
@@ -184,6 +220,7 @@ init_gtk(void)
 	stock_ids = cur->next;
 	g_slist_free_1(cur);
     }
+    
     /* namespace all the gdk stuff in gtk.gdk ... */
     m = Py_InitModule("gtk.gdk", pygdk_functions);
     d = PyModule_GetDict(m);
@@ -211,5 +248,10 @@ init_gtk(void)
     add_atom(SELECTION_TYPE_STRING);
 #undef add_atom
 
+    PyGtkWarning = PyErr_NewException("gtk.GtkWarning", PyExc_Warning, NULL);
+    PyDict_SetItemString(d, "Warning", PyGtkWarning);
+    g_log_set_handler("Gtk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _pygtk_log_func, NULL);
+    
     gtk_timeout_add(100, python_do_pending_calls, NULL);
 }
