@@ -220,11 +220,13 @@ pygtk_generic_tree_model_new(void)
 static guint
 pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -233,14 +235,17 @@ pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
     g_message("get_flags()");
 #endif
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_flags", "");
+    Py_DECREF(self);
     if (py_ret) {
 	guint ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return 0;
     }
@@ -249,12 +254,14 @@ pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
 static gint
 pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, 0);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -263,14 +270,17 @@ pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
     g_message("get_n_columns()");
 #endif
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_n_columns", "");
+    Py_DECREF(self);
     if (py_ret) {
 	gint ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return 0;
     }
@@ -279,11 +289,13 @@ pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
 static GType
 pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, G_TYPE_INVALID);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), G_TYPE_INVALID);
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -293,14 +305,17 @@ pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 #endif
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_column_type",
 				 "(i)", index);
+    Py_DECREF(self);
     if (py_ret) {
 	GType ret = pyg_type_from_object(py_ret);
 
 	Py_DECREF(py_ret);
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return G_TYPE_INVALID;
     }
@@ -310,6 +325,7 @@ static gboolean
 pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
                                   GtkTreeIter *iter, GtkTreePath *path)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_path, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
@@ -318,6 +334,7 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
     g_return_val_if_fail(path != NULL, FALSE);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -328,6 +345,7 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
     py_path = pygtk_tree_path_to_pyobject(path);
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_iter",
 				 "(O)", py_path);
+    Py_DECREF(self);
     Py_DECREF(py_path);
 
     if (py_ret) {
@@ -337,17 +355,20 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	PyErr_Print();
 	iter->user_data = NULL;
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return FALSE;
     }
@@ -356,6 +377,7 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 static GtkTreePath *
 pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, NULL);
@@ -363,6 +385,7 @@ pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_val_if_fail(VALID_ITER(iter, tree_model), NULL);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -376,6 +399,7 @@ pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_path", "(O)",
                                  py_iter);
+    Py_DECREF(self);
     if (py_ret) {
 	GtkTreePath *path = pygtk_tree_path_from_pyobject(py_ret);
 
@@ -383,10 +407,12 @@ pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	    g_warning("could not convert return value of get_path() to "
 		      "a GtkTreePath");
 	Py_DECREF(py_ret);
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return path;
     } else {
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return NULL;
     }
@@ -396,6 +422,7 @@ static void
 pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
                                    gint column, GValue *value)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_value, *py_iter;
 
     g_return_if_fail(tree_model != NULL);
@@ -403,6 +430,7 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
     g_return_if_fail(VALID_ITER(iter, tree_model));
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -420,19 +448,23 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
 	py_iter = Py_None;
     py_value = PyObject_CallMethod(self, METHOD_PREFIX "get_value",
 				   "(Oi)", py_iter,column);
+    Py_DECREF(self);
 
     if (py_value) {
-	pyg_value_from_pyobject(value, py_value);
-	Py_DECREF(py_value);
+        if (py_value != Py_None)
+            pyg_value_from_pyobject(value, py_value);
+        Py_DECREF(py_value);
     } else {
 	PyErr_Print();
     }
+    PyGILState_Release(state);
     pyg_unblock_threads();
 }
 
 static gboolean
 pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
@@ -440,6 +472,7 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_val_if_fail(VALID_ITER(iter, tree_model), FALSE);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -453,6 +486,7 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_next", "(O)",
                                  py_iter);
+    Py_DECREF(self);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
@@ -460,17 +494,20 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
         iter->user_data = NULL;
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return FALSE;
     }
@@ -481,6 +518,7 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
                                        GtkTreeIter *iter,
                                        GtkTreeIter *parent)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_parent = Py_None;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
@@ -489,6 +527,7 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
     g_return_val_if_fail(parent == NULL || parent->stamp == PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp, FALSE);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -500,6 +539,7 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
 	py_parent = (PyObject *)parent->user_data;
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_children",
 				 "(O)", py_parent);
+    Py_DECREF(self);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
@@ -508,17 +548,20 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return FALSE;
     }
@@ -528,6 +571,7 @@ static gboolean
 pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model,
                                         GtkTreeIter *iter)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
@@ -535,6 +579,7 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model,
     g_return_val_if_fail(VALID_ITER(iter, tree_model), FALSE);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -548,14 +593,17 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model,
 
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_has_child",
 				 "(O)", py_iter);
+    Py_DECREF(self);
     if (py_ret) {
 	gboolean ret = PyObject_IsTrue(py_ret);
 
 	Py_DECREF(py_ret);
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return FALSE;
     }
@@ -565,6 +613,7 @@ static gint
 pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model,
                                          GtkTreeIter *iter)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter;
 
     g_return_val_if_fail(tree_model != NULL, 0);
@@ -572,6 +621,7 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model,
     g_return_val_if_fail(iter == NULL || iter->stamp == PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp, 0);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -584,14 +634,17 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model,
 
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_n_children",
 				 "(O)", py_iter);
+    Py_DECREF(self);
     if (py_ret) {
 	gint ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return 0;
     }
@@ -602,6 +655,7 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
                                         GtkTreeIter  *iter,
                                         GtkTreeIter  *parent, gint n)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_parent = Py_None;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
@@ -610,6 +664,7 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
     g_return_val_if_fail(parent == NULL || parent->stamp == PYGTK_GENERIC_TREE_MODEL(tree_model)->stamp, FALSE);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -621,6 +676,7 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
 	py_parent = (PyObject *)parent->user_data;
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_nth_child",
 				 "(Oi)", py_parent, n);
+    Py_DECREF(self);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
@@ -629,17 +685,20 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return FALSE;
     }
@@ -650,6 +709,7 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
                                      GtkTreeIter *iter,
                                      GtkTreeIter *child)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_child = Py_None;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
@@ -658,6 +718,7 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
     g_return_val_if_fail(VALID_ITER(child, tree_model), FALSE);
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -669,6 +730,7 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 	py_child = (PyObject *)child->user_data;
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_parent",
 				 "(O)", py_child);
+    Py_DECREF(self);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
@@ -677,17 +739,20 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    PyGILState_Release(state);
 	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
+	PyGILState_Release(state);
 	pyg_unblock_threads();
 	return FALSE;
     }
@@ -696,6 +761,7 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 static void
 pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter, *method;
 
     g_return_if_fail(tree_model != NULL);
@@ -703,6 +769,7 @@ pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_if_fail(VALID_ITER(iter, tree_model));
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -725,13 +792,16 @@ pygtk_generic_tree_model_unref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	    PyErr_Print();
 	}
     }
+    Py_DECREF(self);
 
+    PyGILState_Release(state);
     pyg_unblock_threads();
 }
 
 static void
 pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
+    PyGILState_STATE state;
     PyObject *self, *py_ret, *py_iter, *method;
 
     g_return_if_fail(tree_model != NULL);
@@ -739,6 +809,7 @@ pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_if_fail(VALID_ITER(iter, tree_model));
 
     pyg_block_threads();
+    state = PyGILState_Ensure();
 
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -761,7 +832,9 @@ pygtk_generic_tree_model_ref_node(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	    PyErr_Print();
 	}
     }
+    Py_DECREF(self);
 
+    PyGILState_Release(state);
     pyg_unblock_threads();
 }
 
